import * as tslib_1 from "tslib";
import { fetch as fetchPolyfill } from 'whatwg-fetch';
var parse = JSON.parse, stringify = JSON.stringify;
var Archer = /** @class */ (function () {
    function Archer() {
        var _this = this;
        // localstorage key
        this.LOCAL_STORAGE_KEY = 'archer-svgs-config'; // 配置
        this.LOCAL_STORAGE_CAHCHE_KEY = 'archer-svgs-cache'; // svg缓存
        // 是否处于预加载状态
        this.isPrefetch = false;
        this.prefetchQueue = []; // svg预加载队列
        // 内存中存储map，避免频繁操作localStorage卡死
        this.svgCache = {};
        // 最大svg 1024kb
        this.maxSize = 1024;
        // 最大缓存个数
        this.max = 100;
        this.getCache = function () {
            var v = localStorage.getItem(_this.LOCAL_STORAGE_CAHCHE_KEY);
            return v ? parse(v) : {};
        };
        this.setCache = function (cache) {
            localStorage.setItem(_this.LOCAL_STORAGE_CAHCHE_KEY, stringify(cache));
        };
        // 从localStorage里获取配置
        this.getCfgFromStorage = function () {
            var v = localStorage.getItem(_this.LOCAL_STORAGE_KEY);
            return parse(v);
        };
        // 设置cfg到localstorage
        this.setCfgToStorage = function (cfg) {
            try {
                localStorage.setItem(_this.LOCAL_STORAGE_KEY, stringify(cfg));
                return true;
            }
            catch (error) {
                console.error(error);
                return false;
            }
        };
        // 设置单例中的配置
        this.init = function (cfg) {
            var localCfg = localStorage.getItem(_this.LOCAL_STORAGE_KEY);
            // 对localStorage中的config进行初始化
            if (!localCfg) {
                _this.setCfgToStorage(cfg);
                _this.localCfg = cfg;
            }
            else {
                _this.localCfg = parse(localCfg);
            }
            _this.config = cfg;
            _this.svgCache = _this.getCache();
        };
        // 开启预下载  一般配置在onload后
        this.startPreFetch = function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var localCfg, keys;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        localCfg = localStorage.getItem(this.LOCAL_STORAGE_KEY);
                        // 对localStorage中的config进行初始化
                        if (!localCfg) {
                            throw (new Error('please init!'));
                        }
                        if (this.isPrefetch) {
                            throw (new Error('prefetching!'));
                        }
                        this.isPrefetch = true;
                        keys = Object.keys(this.config.svgs);
                        if (!(keys.length === 0)) return [3 /*break*/, 1];
                        this.isPrefetch = false;
                        return [2 /*return*/];
                    case 1:
                        this.prefetchQueue = keys;
                        return [4 /*yield*/, this.popQueue()];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3: return [2 /*return*/];
                }
            });
        }); };
        // 下载svg
        this.fetchSvg = function (url) {
            return fetchPolyfill(url).then(function (r) {
                if (r.status === 200) {
                    return r.text();
                }
                return '';
            });
        };
        // svg下载队列队列 - 出队
        this.popQueue = function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.prefetchQueue.length === 0) {
                            this.isPrefetch = false;
                            return [2 /*return*/];
                        }
                        return [4 /*yield*/, this.downloadSvg(this.prefetchQueue[0])];
                    case 1:
                        _a.sent();
                        this.prefetchQueue.shift();
                        return [4 /*yield*/, this.popQueue()];
                    case 2:
                        _a.sent();
                        return [2 /*return*/, true];
                }
            });
        }); };
        // 下载单个svg - 会走localStorage缓存
        this.downloadSvg = function (name) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var localCfg, localSvg, cfgSvg, url, svg, svgsName, size;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        localCfg = this.localCfg;
                        localSvg = localCfg.svgs[name];
                        cfgSvg = this.config.svgs[name];
                        if (!cfgSvg) {
                            this.isPrefetch = false;
                            throw ("no " + name + " svg in config! please check it!");
                        }
                        // svg的信息和地址均匹配
                        if (localSvg.url === cfgSvg.url && localSvg.version === cfgSvg.version) {
                            if (this.svgCache[name] && cfgSvg.cache !== false) {
                                // localstorage里有缓存，直接走缓存
                                return [2 /*return*/, this.svgCache[name]];
                            }
                            else {
                                // 没有缓存，继续往下走，重新下载
                            }
                        }
                        else {
                            // 用cfg的配置覆盖localStorage的配置
                            localCfg.svgs[name] = cfgSvg;
                            this.setCfgToStorage(localCfg);
                        }
                        url = cfgSvg.url;
                        return [4 /*yield*/, this.fetchSvg(url)];
                    case 1:
                        svg = _a.sent();
                        svgsName = Object.keys(this.svgCache);
                        size = (stringify(this.svgCache).length + svg.length) / 1024;
                        // svg图标没有禁用缓存且缓存图标数量没有达到上限才进行缓存
                        if (cfgSvg.cache !== false && (size < this.maxSize || svgsName.indexOf(name) !== -1) && svgsName.length < this.max) {
                            this.svgCache[name] = svg;
                            this.setCache(this.svgCache);
                        }
                        return [2 /*return*/, svg];
                }
            });
        }); };
        // 清除所有svg cache
        this.clearSvgCache = function () {
            localStorage.setItem(_this.LOCAL_STORAGE_CAHCHE_KEY, '{}');
        };
        // 设置svg最大缓存空间
        this.setMaxSize = function (max) {
            _this.maxSize = max;
        };
        // 设置最大缓存个数
        this.setMax = function (max) {
            _this.max = max;
        };
    }
    return Archer;
}());
export default new Archer();
//# sourceMappingURL=Archer.js.map